/******************************************************************************
Project: QTexEngine
License: GNU GPL Version 3 (see LICENSE)
Copyright (C) by the authors:
    2009 Ion Vasilief <ion_vasilief@yahoo.fr>
    2024 Konstantin Kholostov <k.kholostov@fz-juelich.de>
Description: Enables the export of QPainter grafics to .tex files
 ******************************************************************************/

#include "QTeXEngine.h"

#include <QApplication>
#include <QScreen>

QTeXPaintDevice::QTeXPaintDevice(const QString& fileName, const QSize& s, Unit u)
: QPaintDevice()
{
	d_size = s;

	if (!d_size.isValid())
		d_size = QSize(500, 400);

	engine = new QTeXPaintEngine(fileName, u);
}

QTeXPaintDevice::~QTeXPaintDevice()
{
	delete engine;
}

QPaintEngine * QTeXPaintDevice::paintEngine () const
{
	return engine;
}

void QTeXPaintDevice::setColorMode(QPrinter::ColorMode mode)
{
	engine->setGrayScale(mode == QPrinter::GrayScale);
}

void QTeXPaintDevice::setOutputMode(OutputMode mode)
{
	engine->setOutputMode(mode);
}

void QTeXPaintDevice::setUnit(Unit u)
{
	engine->setUnit(u);
}

void QTeXPaintDevice::setDocumentMode(bool on)
{
	engine->setDocumentMode(on);
}

void QTeXPaintDevice::setEscapeTextMode(bool on)
{
	engine->setEscapeTextMode(on);
}

void QTeXPaintDevice::exportFontSizes(bool on)
{
	engine->exportFontSizes(on);
}

void QTeXPaintDevice::setTextHorizontalAlignment(Qt::Alignment alignment)
{
	engine->setTextHorizontalAlignment(alignment);
}

int QTeXPaintDevice::metric ( PaintDeviceMetric metric ) const
{
    QScreen *screen = QGuiApplication::primaryScreen();
    qreal dpi_x = screen->logicalDotsPerInchX();
    qreal dpi_y = screen->logicalDotsPerInchY();
	switch (metric){
		case QPaintDevice::PdmWidth:
			return d_size.width();
		case QPaintDevice::PdmHeight:
			return d_size.height();
		case QPaintDevice::PdmWidthMM:
            return int(25.4 * d_size.width() / dpi_x);
		case QPaintDevice::PdmHeightMM:
            return int(25.4 * d_size.height() / dpi_y);
		case QPaintDevice::PdmNumColors:
			return 65536;//should it be millions?
		case QPaintDevice::PdmDepth:
			return 32;
		case QPaintDevice::PdmDpiX:
		case QPaintDevice::PdmPhysicalDpiX:
			return dpi_x;
		case QPaintDevice::PdmDpiY:
		case QPaintDevice::PdmPhysicalDpiY:
			return dpi_y;
		default:
			qWarning ("QTeXPaintDevice::Unknown metric asked");
			return 0;
	}
}
